
//Set: 1 tab = 2 spaces

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "TSystem.h"
#include "TH1.h"
#include "TH2.h"
#include "TH3.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TChain.h"
#include "TEllipse.h"
#include "TLine.h"
#include "TMarker.h"
#include "TVector3.h"
#include "TRotation.h"
#include "TMath.h"
#include "TROOT.h"
#include "TF1.h"
#include "TGraph.h"
#include "TGraph2D.h"
#include "TGraphErrors.h"
#include "TLatex.h" 
#include "TPolyLine.h" 
#include "TRandom.h"


#define DEUTERON_MASS 1.875614
#define PROTON_MASS 0.93827
#define RAD2DEG 57.295779
#define DEG2RAD 0.017453293
#define deg 0.017453293

using namespace std;

ofstream outdata("TrackOldField.dat");
ofstream outdataBigBite("map_9kGauss_rezultati");
ofstream outdatafield("map_9kGauss_added");
double K = -1.0;
double speed_of_light =299.79; //[mm/ns] 
double T[2][3] = {{0.0, 0.0},{0.0, 0.0}}; // rabimo dve tocki, da bomo lahko potegnili skozi premico;
#define SIGN(a,b) ((b) >= 0 ? fabs(a) : -fabs(a)) 
static double dmaxarg1, dmaxarg2;
#define DMAX(a,b) (dmaxarg1=(a),dmaxarg2=(b),(dmaxarg1) > (dmaxarg2) ? (dmaxarg1) : (dmaxarg2))
static double dminarg1, dminarg2;
#define DMIN(a,b) (dminarg1=(a),dminarg2=(b),(dminarg1) < (dminarg2) ? (dminarg1) : (dminarg2))

int ndata;
const int tock=100000;
double *bxdata = new double [tock];
double *bydata = new double [tock];
double *bzdata = new double [tock];
double *xPts = new double [tock];
double *yPts = new double [tock];
double *zPts = new double [tock];
TVector3 *ptsdata = new TVector3[tock];





void HallA_style() {

  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(508);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.15);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.7);
  gStyle->SetTitleXOffset(1.2);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  gStyle->SetTitleX(0.2);
  gStyle->SetTitleW(0.7);
  gStyle->SetTitleFillColor(18);

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.04,"X");
  gStyle->SetLabelSize(0.04,"Y");
  gStyle->SetNdivisions(508,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);


  gStyle->SetOptStat(1111);

}












double rad(TVector3 p1, TVector3 p2)
//izracuna dolzino vektorja razlike.
{
  TVector3 difference = p1 - p2;
  return difference.Mag();
}


double CalculateFieldComponent(TVector3 pt, Int_t m, TVector3 *points, Double_t *vals, Double_t p = 5.0)
//izracuna eno komponento polja v dani tocki pt.
{
   Double_t pneg = -p;
   Double_t r, w, sum=0.0, sumw=0.0;


   if (fabs(pt.X())>125.0) return 0.0;
   if (pt.Y()<-700.0 || pt.Y()>800.0) return 0.0;
   if (pt.Z()<-400.0 || pt.Y()>1300.0) return 0.0;
     
   for (Int_t i=0; i<m; i++) {   
     if ((r=rad(pt,points[i])) == 0.) return vals[i];
     sum += (w = pow(r,pneg));
     sumw += w*vals[i];
   }
   return sumw/sum;
}

void CalculateField(TVector3 pt, Int_t m, TVector3 *points, Double_t *MapBx, Double_t *MapBy, Double_t *MapBz, TVector3 &Field)
{
  double Bx = CalculateFieldComponent(pt, m, points, MapBx, 5.0);
  double By = CalculateFieldComponent(pt, m, points, MapBy, 5.0);
  double Bz = CalculateFieldComponent(pt, m, points, MapBz, 5.0);
  
  Field.SetXYZ(Bx,By,Bz);
}



void CalculateConstantField(TVector3 pt, double B0, TVector3 &Field)
{
	double alphaB = 20*deg;
	double zC = 662.0;
	double yC = 1.0;
	//double LC = 753.0;
	//double hC = 818.0;
	double low = -479.1;//yC + (zC - LC)/tan(alphaB);
	double high = 591.8;//low + hC;
	//double B0 = 0.92;//0.8*0.92; 
	
	//double x = pt.X(); 
	double y = pt.Y();
	double z = pt.Z();
	
	double Bx = 0.0;
	double By = 0.0;
	double Bz = 0.0; 
	
	if (z<=0) return;
	if (y>(-1.0/tan(alphaB)*(z-zC) + yC)) return;
	if (y<low || y> high) return;

	// dodatni moj (Miha) pogoj, ki ga prej ni bilo
	if (y> (0.3786*z+442.7)) return;
	
	Bx = B0;
  Field.SetXYZ(Bx,By,Bz);
}




int BisFindMin(double searchValue, double *givenArray, int i, int j, int *ijResult){
	if (i==j && givenArray[i]==searchValue) return 1;
	else if ((i>j)  ||  (i==j && givenArray[i]!=searchValue)) {
		//cout<<"bedarija 1"<<endl; 
		return 0;
	}
	if (givenArray[i]!=searchValue) i+=1;	
	int midPoint= (i+j)/2;
	if (searchValue<=givenArray[midPoint]){ijResult[0]=i; ijResult[1]=midPoint;}
	else {ijResult[0]=midPoint; ijResult[1]=j;}
	int iNew=ijResult[0], jNew=ijResult[1];
	return BisFindMin(searchValue, givenArray, iNew, jNew, ijResult);
}

int BisFindMax(double searchValue, double *givenArray, int i, int j, int *ijResult){
	if (i==j && givenArray[i]==searchValue) return 1;
	else if ((i>j)  ||  (i==j && givenArray[i]!=searchValue)) {
		//cout<<"bedarija 2"<<endl; 
		return 0;
	}
	if (givenArray[j]!=searchValue) j-=1;
	int midPoint= (i+j+1)/2;
	if (searchValue>=givenArray[midPoint]){ijResult[0]=midPoint; ijResult[1]=j;}
	else {ijResult[0]=i; ijResult[1]=midPoint;}
	int iNew=ijResult[0], jNew=ijResult[1];
	return BisFindMax(searchValue, givenArray, iNew, jNew, ijResult);
}

int BisFindApprox(double searchValue, double *givenArray, int i, int j, int *ijResult)
{
	if (i>j) {
		//cout<<"bedarija 3"<<endl;
		return 0;
	}
	if (j-i==1 && givenArray[i]<searchValue && searchValue<givenArray[j]) return 2;
	if (j-i==1 && (givenArray[i]>searchValue || searchValue>givenArray[j])) {
		//cout<<"bedarija 4"<<endl;
		return 0;
	}
	int midPoint= (i+j)/2;
	if (searchValue==givenArray[midPoint]) {
		ijResult[0]=midPoint; ijResult[1]=midPoint;
		return 1;
	}
	if (searchValue==givenArray[j]) {
		ijResult[0]=j; ijResult[1]=j;
		return 1;
	}
	if (searchValue<givenArray[midPoint]){ijResult[0]=i; ijResult[1]=midPoint;}
	else {ijResult[0]=midPoint; ijResult[1]=j;}
	int iNew=ijResult[0], jNew=ijResult[1];
	return BisFindApprox(searchValue, givenArray, iNew, jNew, ijResult);

}


int FindPoint(TVector3 pt, int *result){
	int *xijResult=new int[2];	int *yijResult=new int[2];	int *zijResult=new int[2];
	int xApp=BisFindApprox(pt.X(), xPts, 0, ndata-1, xijResult);
	if (xApp!=2) {
		//cout<<"bedarija 5"<<endl;
		return 0;
	}
	if (xApp==2){
		int xMaxOfMin=xijResult[0]; int xMinOfMax=xijResult[1];
		int xMaxOfMax, xMinOfMin;
		if(BisFindMin(xPts[xMaxOfMin], xPts, 0, xMaxOfMin, xijResult)) xMinOfMin=xijResult[0];
		else {
			//cout<<"bedarija 6"<<endl;
			return 0;
		}
		if(BisFindMax(xPts[xMinOfMax], xPts, xMinOfMax, ndata-1, xijResult)) xMaxOfMax=xijResult[1];
		else {
			//cout<<"bedarija 7"<<endl; 
			return 0;
		}


		int yApp1=BisFindApprox(pt.Y(), yPts, xMinOfMin, xMaxOfMin, yijResult);
		if (yApp1==0) {
			//cout<<"bedarija 8 "<<xPts[xMinOfMin]<<" "<<xPts[xMaxOfMin]<<endl; 
			return 0;
		}
		if (yApp1==2){
			int yMaxOfMin1=yijResult[0]; int yMinOfMax1=yijResult[1];
			int yMaxOfMax1, yMinOfMin1;
			if(BisFindMin(yPts[yMaxOfMin1], yPts, xMinOfMin, yMaxOfMin1, yijResult)) yMinOfMin1=yijResult[0];
			else {
				//cout<<"bedarija 9"<<endl; 
				return 0;
			}
			if(BisFindMax(yPts[yMinOfMax1], yPts, yMinOfMax1, xMaxOfMin, yijResult)) yMaxOfMax1=yijResult[1];
			else {
				//cout<<"bedarija 10"<<endl;
				return 0;
			}
			

			int zApp1=BisFindApprox(pt.Z(), zPts, yMinOfMin1, yMaxOfMin1, zijResult);
			if (zApp1!=2) {
				//cout<<"bedarija 11"<<endl; 
				return 0;
			}
			if (zApp1==2){
				result[0]=zijResult[0];
				result[4]=zijResult[1];			
			}

			int zApp2=BisFindApprox(pt.Z(), zPts, yMinOfMax1, yMaxOfMax1, zijResult);
			if (zApp2!=2) {
				//cout<<"bedarija 12"<<endl; 
				return 0;
			}
			if (zApp2==2){
				result[3]=zijResult[0];
				result[7]=zijResult[1];			
			}
		}


		int yApp2=BisFindApprox(pt.Y(), yPts, xMinOfMax, xMaxOfMax, yijResult);
		if (yApp2!=2) {
			//cout<<"bedarija 13"<<endl; 
			return 0;
		}
		if (yApp2==2){
			int yMaxOfMin2=yijResult[0]; int yMinOfMax2=yijResult[1];
			int yMaxOfMax2, yMinOfMin2;
			if(BisFindMin(yPts[yMaxOfMin2], yPts, xMinOfMax, yMaxOfMin2, yijResult)) yMinOfMin2=yijResult[0];
			else {
				//cout<<"bedarija 14"<<endl; 
				return 0;
			}
			if(BisFindMax(yPts[yMinOfMax2], yPts, yMinOfMax2, xMaxOfMax, yijResult)) yMaxOfMax2=yijResult[1];
			else {
				//cout<<"bedarija 15"<<endl; 
				return 0;
			}


			int zApp3=BisFindApprox(pt.Z(), zPts, yMinOfMin2, yMaxOfMin2, zijResult);
			if (zApp3!=2) {
				//cout<<"bedarija 16"<<endl; 
				return 0;
			}
			if (zApp3==2){
				result[1]=zijResult[0];
				result[5]=zijResult[1];			
			}

			int zApp4=BisFindApprox(pt.Z(), zPts, yMinOfMax2, yMaxOfMax2, zijResult);
			if (zApp4!=2) {
				//cout<<"bedarija 17"<<endl;
				return 0;
			}
			if (zApp4==2){
				result[2]=zijResult[0];
				result[6]=zijResult[1];			
			}

		}
	}
	return 1;
}

void CalculateFieldFast(TVector3 pt, TVector3 &Field)
///
//	NOTE: THE FUNCTION WILL RETURN B=(0,0,0) FOR POINTS WITH ANY COORDINATE SAME AS THAT OF ANY KNOWN POINT.
//				ALTHOUGH IT IS STUPID, IT DOESN'T REALLY MATTER, AS WE NEVER REALLY COMPUTE FIELD IN SUCH 'INTEGER' POINTS.
///
{
	int *result=new int[8];
	int PointFound= FindPoint(pt, result);
	double ptBx=0,ptBy=0,ptBz=0,ptx=pt.X(),pty=pt.Y(),ptz=pt.Z();
	//trilinear interpolation.
	if(PointFound){
		//parameters
		double xParam = (ptx-xPts[result[0]]) / (xPts[result[1]]-xPts[result[0]]);
		double yParam = (pty-yPts[result[0]]) / (yPts[result[3]]-yPts[result[0]]);
		double zParam = (ptz-zPts[result[0]]) / (zPts[result[4]]-zPts[result[0]]);
		//given values in known points.
		double bx0=bxdata[result[0]], bx1=bxdata[result[1]],bx2=bxdata[result[2]],bx3=bxdata[result[3]],
						bx4=bxdata[result[4]],bx5=bxdata[result[5]],bx6=bxdata[result[6]],bx7=bxdata[result[7]];
		ptBx= bx0*(1-xParam)*(1-yParam)*(1-zParam) + bx1*xParam*(1-yParam)*(1-zParam) + bx2*xParam*yParam*(1-zParam) + 
					bx3*(1-xParam)*yParam*(1-zParam) + bx4*(1-xParam)*(1-yParam)*zParam + bx5*xParam*(1-yParam)*zParam + 
					bx6*xParam*yParam*zParam + bx7*(1-xParam)*yParam*zParam;
		double by0=bydata[result[0]], by1=bydata[result[1]],by2=bydata[result[2]],by3=bydata[result[3]],
						by4=bydata[result[4]],by5=bydata[result[5]],by6=bydata[result[6]],by7=bydata[result[7]];
		ptBy= by0*(1-xParam)*(1-yParam)*(1-zParam) + by1*xParam*(1-yParam)*(1-zParam) + by2*xParam*yParam*(1-zParam) + 
					by3*(1-xParam)*yParam*(1-zParam) + by4*(1-xParam)*(1-yParam)*zParam + by5*xParam*(1-yParam)*zParam + 
					by6*xParam*yParam*zParam + by7*(1-xParam)*yParam*zParam;
		double bz0=bzdata[result[0]], bz1=bzdata[result[1]],bz2=bzdata[result[2]],bz3=bzdata[result[3]],
						bz4=bzdata[result[4]],bz5=bzdata[result[5]],bz6=bzdata[result[6]],bz7=bzdata[result[7]];
		ptBz= bz0*(1-xParam)*(1-yParam)*(1-zParam) + bz1*xParam*(1-yParam)*(1-zParam) + bz2*xParam*yParam*(1-zParam) + 
					bz3*(1-xParam)*yParam*(1-zParam) + bz4*(1-xParam)*(1-yParam)*zParam + bz5*xParam*(1-yParam)*zParam + 
					bz6*xParam*yParam*zParam + bz7*(1-xParam)*yParam*zParam;
	}
	Field.SetXYZ(ptBx,ptBy,ptBz);
}





void ReadFieldFile(int &nData)
{
	int i=tock/2, j=tock/2+1, k;
	double *Bx = new double [tock];
	double *By = new double [tock];
	double *Bz = new double [tock];
	double *xPtsTemp = new double [tock];
	double *yPtsTemp = new double [tock];
	double *zPtsTemp = new double [tock];
	TVector3 *points = new TVector3[tock];

  ifstream myfile("map_9kGauss");
  if (myfile.is_open())
  {
    double a,b,c,d,e,f;
		int preskok[20];
		int razdelek=0;
		preskok[razdelek]=j;
		razdelek+=1;

		// First we fill positive x values
		while(myfile >> a >> b >> c >> d >> e >> f){
			points[j].SetXYZ(10.0*a,10.0*b,10.0*c); // in [mm]
			xPtsTemp[j] = 10.0*a;
			yPtsTemp[j] = 10.0*b;
			zPtsTemp[j] = 10.0*c;
 	 	  Bx[j] = d*0.1; // in [T]			
 	 	  By[j] = e*0.1;
 	 	  Bz[j] = f*0.1;
			if (j>(tock/2+1)){
				if (xPtsTemp[j]!=xPtsTemp[j-1]){
					preskok[razdelek]=j;
					razdelek+=1;
				}
			}
 	 	  j+=1;
		}
		preskok[razdelek]=j;		

   	// Now we fill negative x values
		for(razdelek=1; razdelek<7; razdelek+=1){
			k=preskok[razdelek]-1;
			while(k>=preskok[razdelek-1]){
   	    points[i].SetXYZ(-points[k].X(),points[k].Y(),points[k].Z());
				xPtsTemp[i] = -points[k].X();
				yPtsTemp[i] = points[k].Y();
				zPtsTemp[i] = points[k].Z();
   	  	Bx[i] = Bx[k];						
   	  	By[i] = -By[k];
   	  	Bz[i] = -Bz[k];
   	  	i-=1;
				k-=1;
			}
    }
    myfile.close();
		
		nData=j-i-1;
		ptsdata=points+i+1;
		xPts=xPtsTemp+i+1;
		yPts=yPtsTemp+i+1;
		zPts=zPtsTemp+i+1;
		bxdata=Bx+i+1;
		bydata=By+i+1;
		bzdata=Bz+i+1;

		/*
		//Let's save this in new file, "map_9kGauss_added".
		for(i=0; i<nData; i+=1)
		{
			outdatafield<<xPts[i]<<"  "<<yPts[i]<<"  "<<zPts[i]<<"  "<<bxdata[i]<<"  "<<bydata[i]<<"  "<<bzdata[i]<<endl;
		}
		outdatafield.close();
		*/
  }
  else cout << "Unable to open file"; 

}



/*///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
Def.: Majka koord. sistem je sistem, kjer:  x kaze noter,
                                            y kaze gor,
                                            z pa desno,
      skladno s sledeco skico [1].
                                _
                               | \
                               |  \
                               ----
                          Skica [1]: BB Coil.

*////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////



/////////////////Pretvorimo podatke z detektorja v Majka koord. sistem./////////////

void TransfOfDet (double *PosSmer, TVector3 &initPos, TVector3 &initSmer)
{
	double xD,yD,zD,vxD,vyD,vzD;
	double iksD=PosSmer[0]*1000,ipsD=PosSmer[1]*1000,thD=PosSmer[2],phD=PosSmer[3]; //in [mm] and [rad].
	zD=1050.0+iksD*sin(25.0*DEG2RAD);
	yD=270.0-iksD*cos(25.0*DEG2RAD);
	xD=ipsD;
	vzD=-cos(thD+25.0*DEG2RAD);		//kar obrnimo ze smer, zdaj bomo simulirali v drugi smeri.
	vyD=-sin(thD+25.0*DEG2RAD);		//enako.
	vxD=-sin(phD);								//enako.
  initPos.SetXYZ(xD,yD,zD);
	initSmer.SetXYZ(vxD,vyD,vzD);
}


















//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////Integrator of particle's trajectory.////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

/* ------- rkck: Take a Cash-Karp Runge-Kutta step:

   Given values for n variables y[1..n] and their derivatives dydx[1..n] 
   known at x, use the fifth-order Cash-Karp Runge-Kutta method to advance 
   the solution over an interval h and return the incremented variables as 
   yout[1..n]. Also return an estimate of the local truncation error in 
   yout using the embedded fourth-order method. 
   The user supplies the routine derivs(x,y,dydx), which returns dervatives 
   dydx at x.
*/

void rkck(double y[], 
	  double dydx[],
	  int n, 
	  double x,
	  double h,
	  double yout[],
	  double yerr[],
	  void (*derivs)(double, double [], double []) )
{
  int i;
  
  // Carsh-Karp Parameters for embedded Runge-Kutta Method:
  static double a2=0.2, a3=0.3, a4=0.6, a5=1.0, a6=0.875, b21=0.2,
    b31=3.0/40.0, b32=9.0/40.0, b41=0.3, b42=-0.9, b43=1.2,
    b51=-11.0/54.0, b52=2.5, b53=-70.0/27.0, b54=35.0/27.0,
    b61=1631.0/55296.0, b62=175.0/512.0, b63=575.0/13824.0,
    b64=44275.0/110592.0, b65=253.0/4096.0, c1=37.0/378.0,
    c3=250.0/621.0, c4=125.0/594.0, c6=512.0/1771.0,
    dc5 = -277.00/14336.0;
  double dc1=c1-2825.0/27648.0, dc3=c3-18578.0/48384.0,
    dc4=c4-13525.0/55296.0, dc6=c6-0.25;

  // alloc temp vectors:
  double *ak2 = new double[n+1];
  double *ak3 = new double[n+1];
  double *ak4 = new double[n+1];
  double *ak5 = new double[n+1];
  double *ak6 = new double[n+1];
  double *ytemp = new double[n+1];

  // ------------------- Fifth order Runge-Kutta (with embedded fourth order):
  // First + second step:
  for (i=1; i<=n; i++)
    ytemp[i] = y[i] + b21*h*dydx[i];
  (*derivs)(x+a2*h,ytemp,ak2);

  // Third step:
  for (i=1; i<=n; i++)
    ytemp[i] = y[i] + h*(b31*dydx[i]+b32*ak2[i]);
  (*derivs)(x+a3*h,ytemp,ak3);	
  
  // Fourth step:
  for (i=1; i<=n; i++)
    ytemp[i] = y[i] + h*(b41*dydx[i]+b42*ak2[i]+b43*ak3[i]);
  (*derivs)(x+a4*h,ytemp,ak4);

  // Fifth step:
  for (i=1; i<=n; i++)
    ytemp[i] = y[i] + h*(b51*dydx[i]+b52*ak2[i]+b53*ak3[i]+b54*ak4[i]);
  (*derivs)(x+a5*h,ytemp,ak5);

  // Sixth step:
  for (i=1; i<=n; i++)
   ytemp[i] = y[i]+h*(b61*dydx[i]+b62*ak2[i]+b63*ak3[i]+b64*ak4[i]+b65*ak5[i]);
  (*derivs)(x+a6*h,ytemp,ak6);

  // Accumulate increments with proper weights:
  for (i=1; i<=n; i++)
    yout[i] = y[i] + h*(c1*dydx[i]+c3*ak3[i]+c4*ak4[i]+c6*ak6[i]);
  // Estimate error as difference between fourth and fifth order methods:
  for (i=1; i<=n; i++)
    yerr[i] = h * (dc1*dydx[i]+dc3*ak3[i]+dc4*ak4[i]+dc5*ak5[i]+dc6*ak6[i]);

  delete[] ytemp;
  delete[] ak6;
  delete[] ak5;
  delete[] ak4;
  delete[] ak3;
  delete[] ak2;

}


#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4


void rkqs(double y[], 
	  double dydx[],
	  int n, 
	  double *x, 
	  double htry, 
	  double eps,
	  double yscal[], 
	  double *hdid,
	  double *hnext,
	  void (*derivs)(double, double [], double []))
{

  int i;
  double errmax,h,htemp,xnew;
  double *yerr  = new double[n+1];
  double *ytemp = new double[n+1];

  // Set stepsize to the initial trial value:
  h=htry;
  for (;;){

    rkck(y,dydx,n,*x,h,ytemp,yerr,derivs);  // Take a step.
    // Evaluate accuracy:
    errmax = 0.0;
    for (i=1; i<=n; i++) errmax = DMAX(errmax, fabs(yerr[i]/yscal[i]));
    errmax /= eps;	        // Scale relative to required tolerance.

    // Step succeeded -> Compute size of next step:
    if (errmax <= 1.0) break;

    // Truncation error too large -> reduce stepsize:
    htemp = SAFETY * h * pow(errmax,PSHRNK);
    h = (h >= 0.0 ? DMAX(htemp, 0.1*h) : DMIN(htemp, 0.1*h));
    // Not more than a factor of 10.
    
    xnew = (*x) + h;
    if ( xnew == *x ) {
      cerr << "ERROR: stepsize underflow (" << h;
      cerr << ") in rkqs" << endl;
      exit(1);
    }
  }
  
  // Compute size of next step:
  if (errmax > ERRCON) *hnext = SAFETY * h * pow(errmax,PGROW);
  else *hnext = 5.0*h;	              // No more than a factor of 5 increase.

  *x += (*hdid=h);
  for (i=1; i<=n; i++) y[i]=ytemp[i];

  delete[] ytemp;
  delete[] yerr;
}

#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON


#define MAXSTP 500000
#define TINY 0.001

int odeint(double ystart[],  // Start values
	    int    nvar,      // number of ODEs (= dimension of the arrays)
	    double x1,        // start value for the independent variable
	    double x2,        // stop at this value of the independent variable
	    double y3min,     // stop if y[3] smaller than y3min
	    double eps,       // required accuracy
	    double h1,        // guessed first stepsize
	    double hmin,      // min. allowed stepsize (can be zero)
	    int    *nok,      // number of good steps taken
	    int    *nbad,     // number of bad (retried and fixed) steps taken
	    double dxsav,     // interval size for step storing
	    int    kmax,      // max number of steps to store
	    int    &kount,    // number of stored steps
	    void (*stepstore)(TGraph*, TGraph*, double, double [], int, int),// func stores steps
	    void (*derivs)(double, double [], double []),  // derivatives
	    double *xact,     // actual value for the independent variable
            TGraph *grYZ,
            TGraph *grXZ )  
{

  
  int nstp,i;
  double xsav=0, x, hnext, hdid, h;

  double *yscal = new double[nvar+1];
  double *y     = new double[nvar+1];
  double *dydx  = new double[nvar+1];
  
  x = x1;
  h = SIGN(h1,x2-x1);
  *nok = (*nbad) = kount = 0;
  for (i=1; i<=nvar; i++) y[i] = ystart[i];
  if (kmax > 0) xsav=x-dxsav*2.0;	// Assures storage of first step.

  for (nstp=1; nstp<=MAXSTP; nstp++) {  // Take at most MAXSTP steps.
    (*derivs)(x,y,dydx);
    for(i=1; i<=nvar; i++)              // Scaling used to monitor accuracy. 
      yscal[i] = fabs(y[i]) + fabs(dydx[i] * h) + TINY;
    // This general-purpose choice can be modified if need be.

    // Store intermediate results:
    if (kmax > 0 && kount < kmax-1 && fabs(x-xsav) > fabs(dxsav)) {
      (*stepstore)(grYZ, grXZ, x, y, nvar, ++kount);
      xsav=x;
    }
    // lf stepsize can overshoot, decrease:
    if ((x+h-x2)*(x+h-x1) > 0.0) h = x2-x;

    rkqs(y, dydx, nvar, &x, h, eps, yscal, &hdid, &hnext, derivs);
    if (hdid == h) ++(*nok); else ++(*nbad);

    // Are we done?
    if ( ((x-x2)*(x2-x1) >= 0.0) ) {
      for (i=1; i<=nvar; i++) ystart[i] = y[i];
      // Save final step:
      if (kmax) (*stepstore)(grYZ, grXZ, x, y, nvar, ++kount);
      

      delete[] dydx; 
      delete[] y;
      delete[] yscal;
      *xact = x;
      return 1;	// Normal exit.
    }
    // Are we done?
    if ( (y[3] < y3min) ) {
      for (i=1; i<=nvar; i++) ystart[i] = y[i];
      // Save final step:
      if (kmax) (*stepstore)(grYZ, grXZ, x, y, nvar, ++kount);
      

      delete[] dydx; 
      delete[] y;
      delete[] yscal;
      *xact = x;
      return 2;	// Normal exit.
    }

    if (fabs(hnext) <= hmin) { 
      cerr << "Step size (" << hnext << ") too small in odeint" << endl;
      return 0;
    }

    h = hnext;
  }
  cerr << "Too many steps in routine odeint" << endl;
  return -1;
}

#undef MAXSTP
#undef TINY


void StoreStep(TGraph *grYZ, TGraph *grXZ, double t,double y[], int ysize, int stepN) 
{
	ysize+=0;
	t+=0;
	//cout<<"Cas: "<<t<<", Lega: ( "<<y[1]<<", "<<y[2]<<", "<<y[3]<<" )"<<endl;
  //outdata<<t<<" "<<y[1]<<" "<<y[2]<<" "<<y[3]<<" "<<y[4]<<" "<<y[5]<<" "<<y[6]<<endl;
  outdata<<y[1]<<" "<<y[2]<<" "<<y[3]<<endl; 
  grYZ->SetPoint(stepN, y[3], y[2]);
  grXZ->SetPoint(stepN, y[3], y[1]);

	//sedaj si se zapomnimo zadnjo tocko in predzadnjo tocko.
	if (stepN>0)
	{
		T[0][0] = T[1][0];
		T[0][1] = T[1][1];
		T[0][2] = T[1][2];
		
		T[1][0] = y[1];
		T[1][1] = y[2];
		T[1][2] = y[3]; 
	}
		
}


void magmove(double t, double y[], double dydx[]) 
{
	t+=0;
	TVector3 Field;
  TVector3 pt(y[1], y[2], y[3]);
  //CalculateField(pt, ndata, ptsdata, bxdata, bydata, bzdata, Field);
	CalculateFieldFast(pt, Field);
	//CalculateConstantField(pt, 0.92, Field);
  double Bx = Field.X(); 
  double By = Field.Y();
  double Bz = Field.Z();
       
  //cout<<"Vrednost Polja: ("<<Bx<<", "<<By<<", "<<Bz<<")"<<endl;
	
	dydx[1] = y[4]; // vx;
	dydx[2] = y[5]; // vy;
	dydx[3] = y[6]; // vz;
	
	//cout<<"Bx  "<<Bx<<endl;

	dydx[4] = K*(y[5]*Bz - y[6]*By);
	dydx[5] = K*(y[6]*Bx - y[4]*Bz);
	dydx[6] = K*(y[4]*By - y[5]*Bx);

}

void Simul(int GraphOn, int coutOn, int OuterGraphOn, TGraph *grYZ, TVector3 initPosition, TVector3 initMomentum, double mass, double charge, TVector3 &PosOnTg)
//mass in MeV/c**2, charge in e_0 (!), initPosition in mm, initMomentum in MeV/c.
{	
	//int N = 0;
	double MWDCalpha = 25.0*deg;
	double MWDCy = 260.0;
	double MWDCz = 1110.0;
  double MWDClength = 1400.0;

  //TGraph *grYZ = new TGraph();
  TGraph *grXZ = new TGraph();
	
	if(coutOn){cout<<"----------------- Big Bite simulation! ---------------------"<<endl;}



	double momentum  = initMomentum.Mag();
	double energy = sqrt(momentum*momentum + mass*mass);
	double gamma = energy/mass;
	K = 89.87551787*charge/(mass*gamma);  //89.87.. = speed_of_light*speed_of_light / 1000
	
	double y[7]; // y gre od 1 do 6 in ne od 0 do 5 kot smo vajeni. 
	// najprej napolnimo koordinate:
	y[1] = initPosition.X();
	y[2] = initPosition.Y();
	y[3] = initPosition.Z();

	// sedaj napolnimo hitrosti
	y[4] = speed_of_light * initMomentum.X()/(mass*gamma);
	y[5] = speed_of_light * initMomentum.Y()/(mass*gamma);
	y[6] = speed_of_light * initMomentum.Z()/(mass*gamma);
	
	double starttime = 0.0;  // Initial time
 	double endtime = 200.0;		//in [ns]
 	int nok, nbad, nstored=0;// Here will be stored numbers of good and bad integration steps  
 	double acttime;

	odeint( y,  		 // Start values
        	6,     		 // number of ODEs (= dimension of the arrays)
        	starttime,       // start value for the independent variable
        	endtime,         // stop at this value of the independent variable
          -1600.0,     	 // stop if y[3] smaller than y3min (mm)
          5e-6,      	 // required accuracy
          1.0,         	 // guessed first stepsize
          1e-20,     	 // min. allowed stepsize (can be zero)
          &nok,      	 // number of good steps taken
          &nbad,    	 // number of bad (retried and fixed) steps taken
          0.2,       	 // interval size for step storing (nsec)
          100000,    	 // max number of steps to store
          nstored,   	 // number of stored steps
          StoreStep, 	 // function which writes the step to file
          magmove,   	 // derivatives function
          &acttime,    	 // actual value for the independent variable
          grYZ,
          grXZ);
		
	double z1,z2,y1,y2,x1,x2;
	x1 = T[0][0];	y1 = T[0][1];	z1 = T[0][2];
	x2 = T[1][0];	y2 = T[1][1];	z2 = T[1][2];
		
		
	////////////////sedaj poiscemo presecisce s Target ravnino.//////////////////
	double kx = (x2 - x1)/(z2 - z1+0.000000001);
	double ky = (y2 - y1)/(z2 - z1+0.000000001);
	double zTg = -1500;
	double yTg = ky*(zTg-z1) + y1;
	double xTg = kx*(zTg-z1) + x1;
	PosOnTg.SetXYZ(xTg,yTg,zTg);
	if(coutOn)
	{
		cout<<"Gibalna: "<<momentum<<" MeV/c"<<endl;
		outdataBigBite<<"Gibalna: "<<momentum<<" MeV/c"<<endl;
		//cout<<"naklon ky: "<<ky<<endl;
		//cout<<"naklon kx: "<<kx<<endl;			
		cout<<"Koordinate na Tg-ravnini: (x,y,z) "<<PosOnTg.X()<<", "<<PosOnTg.Y()<<", "<<PosOnTg.Z()<<endl;
		outdataBigBite<<"Koordinate na Tg-ravnini: (x,y,z) "<<PosOnTg.X()<<", "<<PosOnTg.Y()<<", "<<PosOnTg.Z()<<endl;
	}

  outdata.close();

	if(OuterGraphOn)
	{
    gPad->SetGrid();
    grYZ->SetMarkerStyle(20);
  	grYZ->SetMarkerSize(0.5);
    grYZ->SetMarkerColor(kRed);
    grYZ->GetXaxis()->SetRangeUser(-1700.0, 1600.0);
    grYZ->GetYaxis()->SetRangeUser(-800.0, 1200.0);
		grYZ->GetXaxis()->SetTitle("z [mm]");
		grYZ->GetYaxis()->SetTitle("y [mm]");
  	grYZ->SetTitle("Trajektorija v YZ ravnini.");
  	grYZ->Draw("APL");

    TGraph *grYZOld = new TGraph("CenterLine.dat","%lg %*lg %lg");
    grYZOld->SetMarkerStyle(20);
    grYZOld->SetMarkerSize(0.5);
    grYZOld->SetMarkerColor(kGreen);
    grYZOld->Draw("PL");
       
    double BBz[5] = {0.0, 470.0, 870.0, 0.0, 0.0};
    double BBy[5] = {440.0,  591.8, -479.1 , -479.1, 440.0 };
    TPolyLine *pl1 = new TPolyLine(5, BBz, BBy);
    pl1->SetLineColor(kBlue);
    pl1->SetLineWidth(2);
    pl1->Draw("same");

    double BBYokeZ[7] = {-250.0, 600.0, 900.0, 1000.0, 1290.0, -250.0, -250.0};
    double BBYokeY[7] = {640.0,  960.0, 170.0, 0.0, -760.0, -760.0, 640.0 };
    TPolyLine *plYoke = new TPolyLine(7, BBYokeZ, BBYokeY);
    plYoke->SetLineColor(kOrange);
    plYoke->SetLineWidth(2);
    plYoke->Draw("same");

    double MWDCPointsZ[2] = {MWDCz - MWDClength/2.0*sin(MWDCalpha), MWDCz + MWDClength/2.0*sin(MWDCalpha)};
    double MWDCPointsY[2] = {MWDCy + MWDClength/2.0*cos(MWDCalpha), MWDCy - MWDClength/2.0*cos(MWDCalpha)};
    TPolyLine *plMWDC = new TPolyLine(2, MWDCPointsZ, MWDCPointsY);
    plMWDC->SetLineColor(kViolet);
    plMWDC->SetLineWidth(3);
	  plMWDC->Draw("same");
    
	}


	        	
	if(GraphOn)
	{			
		cout<<"Gen N: "<<grYZ->GetN()<<endl;
    TCanvas *trajectory = new TCanvas("Trajectory", "Trajectory",0,0,900,500);
    trajectory->Divide(2,1);
    trajectory->cd(1);
    gPad->SetGrid();
    grYZ->SetMarkerStyle(20);
  	grYZ->SetMarkerSize(0.5);
    grYZ->SetMarkerColor(kRed);
    grYZ->GetXaxis()->SetRangeUser(-1700.0, 1600.0);
    grYZ->GetYaxis()->SetRangeUser(-800.0, 1200.0);
		grYZ->GetXaxis()->SetTitle("z [mm]");
		grYZ->GetYaxis()->SetTitle("y [mm]");
  	grYZ->SetTitle("Trajektorija v YZ ravnini.");
  	grYZ->Draw("APL");

    TGraph *grYZOld = new TGraph("CenterLine.dat","%lg %*lg %lg");
    grYZOld->SetMarkerStyle(20);
    grYZOld->SetMarkerSize(0.5);
    grYZOld->SetMarkerColor(kGreen);
    grYZOld->Draw("PL");
       
    double BBz[5] = {0.0, 470.0, 870.0, 0.0, 0.0};
    double BBy[5] = {440.0,  591.8, -479.1 , -479.1, 440.0 };
    TPolyLine *pl1 = new TPolyLine(5, BBz, BBy);
    pl1->SetLineColor(kBlue);
    pl1->SetLineWidth(2);
    pl1->Draw("same");

    double BBYokeZ[7] = {-250.0, 600.0, 900.0, 1000.0, 1290.0, -250.0, -250.0};
    double BBYokeY[7] = {640.0,  960.0, 170.0, 0.0, -760.0, -760.0, 640.0 };
    TPolyLine *plYoke = new TPolyLine(7, BBYokeZ, BBYokeY);
    plYoke->SetLineColor(kOrange);
    plYoke->SetLineWidth(2);
    plYoke->Draw("same");

    double MWDCPointsZ[2] = {MWDCz - MWDClength/2.0*sin(MWDCalpha), MWDCz + MWDClength/2.0*sin(MWDCalpha)};
    double MWDCPointsY[2] = {MWDCy + MWDClength/2.0*cos(MWDCalpha), MWDCy - MWDClength/2.0*cos(MWDCalpha)};
    TPolyLine *plMWDC = new TPolyLine(2, MWDCPointsZ, MWDCPointsY);
    plMWDC->SetLineColor(kViolet);
    plMWDC->SetLineWidth(3);
	  plMWDC->Draw("same");

    trajectory->cd(2);
    gPad->SetGrid();
    grXZ->SetMarkerStyle(20);
    grXZ->SetMarkerSize(0.5);
    grXZ->SetMarkerColor(kRed);
    grXZ->GetXaxis()->SetRangeUser(-1700.0, 1600.0);
    grXZ->GetYaxis()->SetRangeUser(-600.0, 600.0);
  	grXZ->GetXaxis()->SetTitle("z [mm]");
		grXZ->GetYaxis()->SetTitle("x [mm]");
  	grXZ->SetTitle("Trajektorija v XZ ravnini.");
    grXZ->Draw("APL");
    
    trajectory->Delete();
	}


}






//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


















void BBinverse(){

  HallA_style();

//////////////////////////////////////////////////
//Read FIELD data and store them.
	ReadFieldFile(ndata);
	cout <<"Stevilo tock: "<<ndata<<endl;
//////////////////////////////////////////////////




///////////////////////////////////////////////////
//Read data from DETECTOR and store them.

  TChain *particlesOnDet =new TChain("T");
  TString filename = Form("detektor.root");
  particlesOnDet->Add(filename.Data());
  assert(particlesOnDet);

	int freqN,i,j,eventCounter=0, num=particlesOnDet->GetEntries();
	const int maxndata=10;			//if 'multiple' events occur.
	const int absmaxdata=1200;	//all events together.
	double iks[maxndata],ips[maxndata],th[maxndata],ph[maxndata],PosMom[absmaxdata][4];
  particlesOnDet->SetBranchAddress("BB.tr.x", &iks);
  particlesOnDet->SetBranchAddress("BB.tr.y", &ips);
  particlesOnDet->SetBranchAddress("BB.tr.th", &th);
  particlesOnDet->SetBranchAddress("BB.tr.ph", &ph);
  particlesOnDet->SetBranchAddress("Ndata.BB.tr.x", &freqN);

/*
///////////////////////////////////////////////////
  //Histograms for data (x,y,th,ph) representation.

  cout<<"Number of entries : "<<num<<endl;
  TH1F *h1 = new TH1F("h","h",200,-0.8,0.8);
  TH1F *h2 = new TH1F("h","h",200,-0.8,0.8);
  TH1F *h3 = new TH1F("h","h",200,-0.8,0.8);
  TH1F *h4 = new TH1F("h","h",200,-0.8,0.8);	

*/

  for (i=0; i<num; i++)
  {
    particlesOnDet->GetEntry(i);
		for(j=0; j<freqN; j++){
    	//h1->Fill(iks[j]); h2->Fill(ips[j]); h3->Fill(th[j]); h4->Fill(ph[j]);
			PosMom[eventCounter][0]=iks[j];
			PosMom[eventCounter][1]=ips[j];
			PosMom[eventCounter][2]=th[j];
			PosMom[eventCounter][3]=ph[j];
			eventCounter++;
		}
  }

  delete particlesOnDet;
	particlesOnDet=NULL;



/*
  //Histograms for data (x,y,th,ph) representation.

  TCanvas *c0 = new TCanvas("c0","Raw Coincidence Time Plots",800,800);
  c0->Divide(2,2);
  c0->cd(1);
  h1->Draw();
  h1->GetXaxis()->SetTitle("x [m]");
  h1->SetTitle("BB.tr.x");
  c0->cd(2);
  h2->Draw();
  h2->GetXaxis()->SetTitle("y [m]");
  h2->SetTitle("BB.tr.y");
  c0->cd(3);
  h3->Draw();
  h3->GetXaxis()->SetTitle("th [rd]");
  h3->SetTitle("BB.tr.th");
  c0->cd(4);
  h4->Draw();
  h4->GetXaxis()->SetTitle("ph [rd]");
  h4->SetTitle("BB.tr.ph");

*/








/*
//////////////////////test point //////////
	TVector3 testPoint(0.51,15.01,0.01);
	TVector3 fieldInTestPoint;
	CalculateFieldFast(testPoint, fieldInTestPoint);
	cout<<"Testna tocka (fast):   Bx: "<<fieldInTestPoint(0)<<"  By: "<<fieldInTestPoint(1)<<" Bz: "<<fieldInTestPoint(2)<<endl;
	CalculateField(testPoint, ndata, ptsdata, bxdata, bydata, bzdata, fieldInTestPoint);
	cout<<"Testna tocka (p=5):   Bx: "<<fieldInTestPoint(0)<<"  By: "<<fieldInTestPoint(1)<<" Bz: "<<fieldInTestPoint(2)<<endl;

	int *result=new int[8];
	if(FindPoint(testPoint, result)) cout<<"Indeksi: "<<result[0]<<" "<<result[1]<<" "<<result[2]<<" "<<result[3]<<" "<<result[4]<<" "<<result[5]<<" "<<result[6]<<" "<<result[7]<<endl;

////////////////////////////////////////////
*/









///////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
////////////////Finding the momentum with shooting method.
////////////////
/////////////////////////////////////////////////////////////////


	double xTgSize=10.0, yTgSize=10.0, maxPhiAngle=atan(0.2/1.5);
	int GoodOne=0, stepsInBis=20;
	TGraph *graphYZ = new TGraph();
	int ii,jj;
  TVector3 initPos;
  TVector3 initMom;
	TVector3 PosOnTg;

	//TCanvas *trajEvolution = new TCanvas("Evolution of Trajectories", "Evolution of Trajectories",0,0,950,900);
	//trajEvolution->Divide(3,3);

  for (ii=0; ii<eventCounter; ii++)
  {
		cout<<"----------------------------------------------------"<<endl;
		cout<<"-------------------------"<<ii+1<<"-------------------------"<<endl;
		cout<<"----------------------------------------------------"<<endl;
		outdataBigBite<<"----------------------------------------------------"<<endl;
		outdataBigBite<<"-------------------------"<<ii+1<<"-------------------------"<<endl;
		outdataBigBite<<"----------------------------------------------------"<<endl;

		TransfOfDet (PosMom[ii], initPos, initMom);

		if(fabs(PosMom[ii][3]) > maxPhiAngle){
			cout<<"Ni pravi delec (kot fi)."<<endl;
			outdataBigBite<<"Ni pravi delec (kot fi)."<<endl;
		}
		initMom*=(1.0/initMom.Mag());
		double mom1=200.0, mom2=1300.0;
		double mom=(mom1+mom2)/2.0;
		initMom*=mom1;
		//trajEvolution->cd(1);
		TGraph *graphYZ1 = new TGraph();
		Simul(0,0,0, graphYZ1,initPos, initMom, 938.27, -1.0, PosOnTg);
		//cout<<"po prvi simulaciji"<<endl;
		if (PosOnTg.Y()<0){
			cout<<"Ni pravi delec (zagotovo bi prisel izpod tarce)."<<endl;
			outdataBigBite<<"Ni pravi delec (zagotovo bi prisel izpod tarce)."<<endl;
			continue;
		}
		initMom*=(1.0/mom1);
		initMom*=mom2;
		//trajEvolution->cd(2);
		TGraph *graphYZ2 = new TGraph();
		Simul(0,0,0, graphYZ2,initPos, initMom, 938.27, -1.0, PosOnTg);
		//cout<<"po drugi simulaciji"<<endl;
		if (PosOnTg.Y()>0){
			cout<<"Ni pravi delec (zagotovo bi prisel iznad tarce)."<<endl;
			outdataBigBite<<"Ni pravi delec (zagotovo bi prisel iznad tarce)."<<endl;
			continue;
		}
		initMom*=(1.0/mom2);
		GoodOne=0;
		for(jj=0; jj<stepsInBis; jj++){
			mom=(mom1+mom2)/2.0;
			initMom*=mom;
			//trajEvolution->cd(3+jj);
			//TGraph *graphYZ = new TGraph();
			Simul(0,0,0, graphYZ,initPos, initMom, 938.27, -1.0, PosOnTg);
			initMom*=(1.0/mom);
			//cout<<"po "<<jj+2<<". simulaciji"<<endl;
			if (fabs(PosOnTg.Y())<yTgSize /*&& fabs(PosOnTg.X())<xTgSize */){
				GoodOne=1;
				break;
			}
			if (PosOnTg.Y()>=0) mom1=mom;
			else mom2=mom;
		}
		initMom*=mom;
		Simul(0,1,0, graphYZ,initPos, initMom, 938.27, -1.0, PosOnTg);
		initMom*=(1.0/mom);
		if(GoodOne) {
			cout<<"Zadeli smo tarco v "<<jj+2<<". koraku pri gibalni kolicini "<<mom<<" MeV/c."<<endl;
			outdataBigBite<<"Zadeli smo tarco v "<<jj+2<<". koraku pri gibalni kolicini "<<mom<<" MeV/c."<<endl;
			if (fabs(PosOnTg.X()) > xTgSize){
				cout<<"Pri tem je koordinata X = "<<PosOnTg.X()<<" mm."<<endl;
				outdataBigBite<<"Pri tem je koordinata X = "<<PosOnTg.X()<<" mm."<<endl;
			}
		}
		else{
			cout<<"Tarce tudi po "<<stepsInBis+1<<" korakih nismo zadeli!"<<endl;
			outdataBigBite<<"Tarce tudi po "<<stepsInBis+1<<" korakih nismo zadeli!"<<endl;
		}
  }


//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////




/*

//////////////////////////////////////////////////////////////
///////Now we show why shooting method is appropriate.////////
//////////////////////////////////////////////////////////////


	TCanvas *trajectories = new TCanvas("Trajectories", "Why shooting method is good?",0,0,950,900);
	trajectories->Divide(3,3);

	TransfOfDet (PosMom[2], initPos, initMom);
	initMom*=(1.0/initMom.Mag());

	for (ii=1; ii<=9; ii+=1){
		TGraph *gTempYZ = new TGraph();
		trajectories->cd(ii);
		Simul(0,0,1, gTempYZ,initPos, initMom*(150+50*ii), 938.27, -1.0, PosOnTg);
		char *gTempTitle=new char[100];
		int appended= sprintf (gTempTitle, "Trajektorija v YZ ravnini, \npri gibalni kolicini %d MeV.", 150+50*ii);
		if(appended>0){
			const char *gYZTitle=gTempTitle;
	  	gTempYZ->SetTitle(gYZTitle);
		}
	}

//////////////////////////////////////////////////////////////////7
*/
























	

/*

///////////////////////////////////////////////////////////////////////////////
///////////////Graphs for field representation.//////////////////////////////////

  Int_t numOfPoints = 70;
	Int_t tockaNaGrafu=0;
  Double_t vrednost, xx, yy, zz;
  TGraph2D *dt1 = new TGraph2D();
	TGraph2D *dt2 = new TGraph2D();
	TGraph2D *dt3 = new TGraph2D();
	TGraph2D *dt4 = new TGraph2D();
	TGraph2D *dt5 = new TGraph2D();
	TGraph2D *dt6 = new TGraph2D();
	TVector3 TempField;
  for (Int_t N=0; N<numOfPoints; N++) {
		for (Int_t M=0; M<numOfPoints; M++){
			//Ravnina x=0.
			//Graf Bx.
    	yy = (((double)N)/numOfPoints)*1600-750+0.5;
    	zz = (((double)M)/numOfPoints)*1800-450+0.5;
			//Vrednost vsakic izracunamo.
			TVector3 nasaTocka(0.01,yy,zz);
			//vrednost=CalculateFieldComponent(nasaTocka, ndata, ptsdata, bxdata, 5.0);
			CalculateFieldFast(nasaTocka,TempField);
			vrednost=TempField.X();
    	dt1->SetPoint(tockaNaGrafu,zz,yy,vrednost);

			//Graf By.
			yy = (((double)N)/numOfPoints)*1600-750+0.5;
    	zz = (((double)M)/numOfPoints)*1800-450+0.5;
			nasaTocka.SetX(5.01);
			//Vrednost vsakic izracunamo.
			//vrednost=CalculateFieldComponent(nasaTocka, ndata, ptsdata, bydata, 5.0);
			CalculateFieldFast(nasaTocka,TempField);
			vrednost=TempField.Y();
    	dt2->SetPoint(tockaNaGrafu,zz,yy,vrednost);

			//Graf Bz.
			yy = (((double)N)/numOfPoints)*1600-750+0.5;
    	zz = (((double)M)/numOfPoints)*1800-450+0.5;
			//Vrednost vsakic izracunamo.
			//vrednost=CalculateFieldComponent(nasaTocka, ndata, ptsdata, bzdata, 5.0);
			CalculateFieldFast(nasaTocka,TempField);
			vrednost=TempField.Z();
    	dt3->SetPoint(tockaNaGrafu,zz,yy,vrednost);
			tockaNaGrafu+=1;

			//Ravnina z=0.
			//Graf Bz.
			xx = (((double)N)/numOfPoints)*500-250+0.5;
    	yy = (((double)M)/numOfPoints)*1600-750+0.5;
			//Vrednost vsakic izracunamo.
			TVector3 novaTocka(xx,yy,0.01);
			//vrednost=CalculateFieldComponent(novaTocka, ndata, ptsdata, bzdata, 5.0);
			CalculateFieldFast(novaTocka,TempField);
			vrednost=TempField.Z();
    	dt4->SetPoint(tockaNaGrafu,xx,yy,vrednost);

			//Graf Bx na z=0.
			xx = (((double)N)/numOfPoints)*250-125+0.5;
    	yy = (((double)M)/numOfPoints)*1600-750+0.5;
			//Vrednost vsakic izracunamo.
			//vrednost=CalculateFieldComponent(novaTocka, ndata, ptsdata, bxdata, 5.0);
			CalculateFieldFast(novaTocka,TempField);
			vrednost=TempField.X();
    	dt5->SetPoint(tockaNaGrafu,xx,yy,vrednost);

			//Graf By na y=0.
			xx = (((double)N)/numOfPoints)*250-125+0.5;
    	zz = (((double)M)/numOfPoints)*1800-450+0.5;
			//Vrednost vsakic izracunamo.
			nasaTocka.SetXYZ(xx,0.01,zz);
			CalculateFieldFast(nasaTocka,TempField);
			vrednost=TempField.Y();
    	dt6->SetPoint(tockaNaGrafu,xx,zz,vrednost);

		}
  }

	///////////Let's draw field components in different planes.///////
	//////////////////////////////////////////////////////////////////

  TCanvas *bBite = new TCanvas("BigBite","BigBite polje v ravnini",0,0,600,900);
  bBite->Divide(2,3);
  gStyle->SetPalette(1);
	bBite->cd(1);
  //dt1->Draw("surf1");
  dt1->Draw("colz");
  dt1->GetXaxis()->SetTitle("z [mm]");
	dt1->GetYaxis()->SetTitle("y [mm]");
  dt1->SetTitle("Bx v ravnini x=0");	
	bBite->cd(2);
  //dt2->Draw("surf1");
  dt2->Draw("colz");
  dt2->GetXaxis()->SetTitle("z [mm]");
	dt2->GetYaxis()->SetTitle("y [mm]");
  dt2->SetTitle("By v ravnini x=5");
	bBite->cd(3);
  //dt3->Draw("surf1");
  dt3->Draw("colz");
  dt3->GetXaxis()->SetTitle("z [mm]");
	dt3->GetYaxis()->SetTitle("y [mm]");
  dt3->SetTitle("Bz v ravnini x=5");
	bBite->cd(4);
  //dt4->Draw("surf1");
  dt4->Draw("colz");
  dt4->GetXaxis()->SetTitle("x [mm]");
	dt4->GetYaxis()->SetTitle("y [mm]");
  dt4->SetTitle("Bz v ravnini z=0");
	bBite->cd(5);
  //dt5->Draw("surf1");
  dt5->Draw("colz");
  dt5->GetXaxis()->SetTitle("x [mm]");
	dt5->GetYaxis()->SetTitle("y [mm]");
  dt5->SetTitle("Bx v ravnini z=0");
	bBite->cd(6);
  //dt6->Draw("surf1");
  dt6->Draw("colz");
  dt6->GetXaxis()->SetTitle("x [mm]");
	dt6->GetYaxis()->SetTitle("z [mm]");
  dt6->SetTitle("By v ravnini y=0");

*/









	outdata.close();
	outdataBigBite.close();
	outdatafield.close();

}//end


